@@ -16,11 +16,11 @@
 """Manage access to the clients, including authenticating when needed."""
 
 import logging
-import pkg_resources
 import sys
 
 from osc_lib import clientmanager
 from osc_lib import shell
+import pkg_resources
 
 
 LOG = logging.getLogger(__name__)
@@ -91,13 +91,6 @@ class ClientManager(clientmanager.ClientManager):
 
         return super(ClientManager, self).setup_auth()
 
-    @property
-    def auth_ref(self):
-        if not self._auth_required:
-            return None
-        else:
-            return super(ClientManager, self).auth_ref
-
     def _fallback_load_auth_plugin(self, e):
         # NOTES(RuiChen): Hack to avoid auth plugins choking on data they don't
         #                 expect, delete fake token and endpoint, then try to
@@ -125,6 +118,25 @@ class ClientManager(clientmanager.ClientManager):
         #                use Network API by default
         return self.is_service_available('network') is not False
 
+    def is_compute_endpoint_enabled(self):
+        """Check if Compute endpoint is enabled"""
+
+        return self.is_service_available('compute') is not False
+
+    def is_volume_endpoint_enabled(self, volume_client):
+        """Check if volume endpoint is enabled"""
+        # NOTE(jcross): Cinder did some interesting things with their service
+        #               name so we need to figure out which version to look
+        #               for when calling is_service_available()
+        volume_version = volume_client.api_version.ver_major
+        if self.is_service_available(
+                "volumev%s" % volume_version) is not False:
+            return True
+        elif self.is_service_available('volume') is not False:
+            return True
+        else:
+            return False
+
 
 # Plugin Support
 
@@ -132,9 +144,15 @@ def get_plugin_modules(group):
     """Find plugin entry points"""
     mod_list = []
     for ep in pkg_resources.iter_entry_points(group):
-        LOG.debug('Found plugin %r', ep.name)
+        LOG.debug('Found plugin %s', ep.name)
+
+        try:
+            __import__(ep.module_name)
+        except Exception:
+            sys.stderr.write(
+                "WARNING: Failed to import plugin %s.\n" % ep.name)
+            continue
 
-        __import__(ep.module_name)
         module = sys.modules[ep.module_name]
         mod_list.append(module)
         init_func = getattr(module, 'Initialize', None)
