@@ -17,11 +17,11 @@
 
 import logging
 
+from novaclient import api_versions
 from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
-import six
 
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
@@ -134,6 +134,12 @@ class CreateFlavor(command.ShowOne):
             help=_("Allow <project> to access private flavor (name or ID) "
                    "(Must be used with --private option)"),
         )
+        parser.add_argument(
+            '--description',
+            metavar='<description>',
+            help=_("Description for the flavor.(Supported by API versions "
+                   "'2.55' - '2.latest'")
+        )
         identity_common.add_project_domain_option_to_parser(parser)
         return parser
 
@@ -145,6 +151,11 @@ class CreateFlavor(command.ShowOne):
             msg = _("--project is only allowed with --private")
             raise exceptions.CommandError(msg)
 
+        if parsed_args.description:
+            if compute_client.api_version < api_versions.APIVersion("2.55"):
+                msg = _("--os-compute-api-version 2.55 or later is required")
+                raise exceptions.CommandError(msg)
+
         args = (
             parsed_args.name,
             parsed_args.ram,
@@ -154,7 +165,8 @@ class CreateFlavor(command.ShowOne):
             parsed_args.ephemeral,
             parsed_args.swap,
             parsed_args.rxtx_factor,
-            parsed_args.public
+            parsed_args.public,
+            parsed_args.description
         )
 
         flavor = compute_client.flavors.create(*args)
@@ -182,7 +194,7 @@ class CreateFlavor(command.ShowOne):
         flavor_info.pop("links")
         flavor_info['properties'] = utils.format_dict(flavor.get_keys())
 
-        return zip(*sorted(six.iteritems(flavor_info)))
+        return zip(*sorted(flavor_info.items()))
 
 
 class DeleteFlavor(command.Command):
@@ -332,6 +344,12 @@ class SetFlavor(command.Command):
             help=_('Set flavor access to project (name or ID) '
                    '(admin only)'),
         )
+        parser.add_argument(
+            '--description',
+            metavar='<description>',
+            help=_("Set description for the flavor.(Supported by API "
+                   "versions '2.55' - '2.latest'")
+        )
         identity_common.add_project_domain_option_to_parser(parser)
 
         return parser
@@ -380,6 +398,13 @@ class SetFlavor(command.Command):
             raise exceptions.CommandError(_("Command Failed: One or more of"
                                             " the operations failed"))
 
+        if parsed_args.description:
+            if compute_client.api_version < api_versions.APIVersion("2.55"):
+                msg = _("--os-compute-api-version 2.55 or later is required")
+                raise exceptions.CommandError(msg)
+            compute_client.flavors.update(flavor=parsed_args.flavor,
+                                          description=parsed_args.description)
+
 
 class ShowFlavor(command.ShowOne):
     _description = _("Display flavor details")
@@ -406,7 +431,7 @@ class ShowFlavor(command.ShowOne):
                 projects = [utils.get_field(access, 'tenant_id')
                             for access in flavor_access]
                 # TODO(Huanxuan Ao): This format case can be removed after
-                # patch https://review.openstack.org/#/c/330223/ merged.
+                # patch https://review.opendev.org/#/c/330223/ merged.
                 access_projects = utils.format_list(projects)
             except Exception as e:
                 msg = _("Failed to get access projects list "
@@ -421,7 +446,7 @@ class ShowFlavor(command.ShowOne):
 
         flavor['properties'] = utils.format_dict(resource_flavor.get_keys())
 
-        return zip(*sorted(six.iteritems(flavor)))
+        return zip(*sorted(flavor.items()))
 
 
 class UnsetFlavor(command.Command):
