@@ -68,6 +68,25 @@ def find_service(identity_client, name_type_or_id):
         raise exceptions.CommandError(msg % name_type_or_id)
 
 
+def get_resource(manager, name_type_or_id):
+    # NOTE (vishakha): Due to bug #1799153 and for any another related case
+    # where GET resource API does not support the filter by name,
+    # osc_lib.utils.find_resource() method cannot be used because that method
+    # try to fall back to list all the resource if requested resource cannot
+    # be get via name. Which ends up with NoUniqueMatch error.
+    # This new function is the replacement for osc_lib.utils.find_resource()
+    # for resources does not support GET by name.
+    # For example: identity GET /regions.
+    """Find a resource by id or name."""
+
+    try:
+        return manager.get(name_type_or_id)
+    except identity_exc.NotFound:
+        # raise NotFound exception
+        msg = _("No resource with name or id of '%s' exists")
+        raise exceptions.CommandError(msg % name_type_or_id)
+
+
 def _get_token_resource(client, resource, parsed_name, parsed_domain=None):
     """Peek into the user's auth token to get resource IDs
 
@@ -101,6 +120,13 @@ def _get_token_resource(client, resource, parsed_name, parsed_domain=None):
         # user/project under different domain may has a same name
         if parsed_domain and parsed_domain not in obj['domain'].values():
             return parsed_name
+        if isinstance(obj, list):
+            for item in obj:
+                if item['name'] == parsed_name:
+                    return item['id']
+                if item['id'] == parsed_name:
+                    return parsed_name
+            return parsed_name
         return obj['id'] if obj['name'] == parsed_name else parsed_name
     # diaper defense in case parsing the token fails
     except Exception:  # noqa
@@ -181,7 +207,7 @@ def _find_identity_resource(identity_client_manager, name_or_id,
                                                 name_or_id, **kwargs)
         if identity_resource is not None:
             return identity_resource
-    except identity_exc.Forbidden:
+    except exceptions.Forbidden:
         pass
 
     return resource_type(None, {'id': name_or_id, 'name': name_or_id})
@@ -207,13 +233,13 @@ def add_group_domain_option_to_parser(parser):
     )
 
 
-def add_project_domain_option_to_parser(parser):
+def add_project_domain_option_to_parser(parser, enhance_help=lambda _h: _h):
     parser.add_argument(
         '--project-domain',
         metavar='<project-domain>',
-        help=_('Domain the project belongs to (name or ID). '
-               'This can be used in case collisions between project names '
-               'exist.'),
+        help=enhance_help(_('Domain the project belongs to (name or ID). This '
+                            'can be used in case collisions between project '
+                            'names exist.')),
     )
 
 
