@@ -15,41 +15,45 @@
 
 """Identity v3 Domain action implementations"""
 
-import six
-import sys
+import logging
 
 from keystoneauth1 import exceptions as ks_exc
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
 
-from openstackclient.common import command
-from openstackclient.common import utils
-from openstackclient.i18n import _  # noqa
+from openstackclient.i18n import _
+from openstackclient.identity import common
+
+
+LOG = logging.getLogger(__name__)
 
 
 class CreateDomain(command.ShowOne):
-    """Create new domain"""
+    _description = _("Create new domain")
 
     def get_parser(self, prog_name):
         parser = super(CreateDomain, self).get_parser(prog_name)
         parser.add_argument(
             'name',
             metavar='<domain-name>',
-            help='New domain name',
+            help=_('New domain name'),
         )
         parser.add_argument(
             '--description',
             metavar='<description>',
-            help='New domain description',
+            help=_('New domain description'),
         )
         enable_group = parser.add_mutually_exclusive_group()
         enable_group.add_argument(
             '--enable',
             action='store_true',
-            help='Enable domain (default)',
+            help=_('Enable domain (default)'),
         )
         enable_group.add_argument(
             '--disable',
             action='store_true',
-            help='Disable domain',
+            help=_('Disable domain'),
         )
         parser.add_argument(
             '--or-show',
@@ -71,39 +75,52 @@ class CreateDomain(command.ShowOne):
                 description=parsed_args.description,
                 enabled=enabled,
             )
-        except ks_exc.Conflict as e:
+        except ks_exc.Conflict:
             if parsed_args.or_show:
                 domain = utils.find_resource(identity_client.domains,
                                              parsed_args.name)
-                self.log.info('Returning existing domain %s', domain.name)
+                LOG.info(_('Returning existing domain %s'), domain.name)
             else:
-                raise e
+                raise
 
         domain._info.pop('links')
-        return zip(*sorted(six.iteritems(domain._info)))
+        return zip(*sorted(domain._info.items()))
 
 
 class DeleteDomain(command.Command):
-    """Delete domain"""
+    _description = _("Delete domain(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteDomain, self).get_parser(prog_name)
         parser.add_argument(
             'domain',
             metavar='<domain>',
-            help='Domain to delete (name or ID)',
+            nargs='+',
+            help=_('Domain(s) to delete (name or ID)'),
         )
         return parser
 
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
-        domain = utils.find_resource(identity_client.domains,
-                                     parsed_args.domain)
-        identity_client.domains.delete(domain.id)
+        result = 0
+        for i in parsed_args.domain:
+            try:
+                domain = utils.find_resource(identity_client.domains, i)
+                identity_client.domains.delete(domain.id)
+            except Exception as e:
+                result += 1
+                LOG.error(_("Failed to delete domain with name or "
+                          "ID '%(domain)s': %(e)s"), {'domain': i, 'e': e})
+
+        if result > 0:
+            total = len(parsed_args.domain)
+            msg = (_("%(result)s of %(total)s domains failed "
+                   "to delete.") % {'result': result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListDomain(command.Lister):
-    """List domains"""
+    _description = _("List domains")
 
     def take_action(self, parsed_args):
         columns = ('ID', 'Name', 'Enabled', 'Description')
@@ -116,35 +133,35 @@ class ListDomain(command.Lister):
 
 
 class SetDomain(command.Command):
-    """Set domain properties"""
+    _description = _("Set domain properties")
 
     def get_parser(self, prog_name):
         parser = super(SetDomain, self).get_parser(prog_name)
         parser.add_argument(
             'domain',
             metavar='<domain>',
-            help='Domain to modify (name or ID)',
+            help=_('Domain to modify (name or ID)'),
         )
         parser.add_argument(
             '--name',
             metavar='<name>',
-            help='New domain name',
+            help=_('New domain name'),
         )
         parser.add_argument(
             '--description',
             metavar='<description>',
-            help='New domain description',
+            help=_('New domain description'),
         )
         enable_group = parser.add_mutually_exclusive_group()
         enable_group.add_argument(
             '--enable',
             action='store_true',
-            help='Enable domain',
+            help=_('Enable domain'),
         )
         enable_group.add_argument(
             '--disable',
             action='store_true',
-            help='Disable domain',
+            help=_('Disable domain'),
         )
         return parser
 
@@ -163,28 +180,29 @@ class SetDomain(command.Command):
         if parsed_args.disable:
             kwargs['enabled'] = False
 
-        if not kwargs:
-            sys.stdout.write("Domain not updated, no arguments present")
-            return
         identity_client.domains.update(domain.id, **kwargs)
 
 
 class ShowDomain(command.ShowOne):
-    """Display domain details"""
+    _description = _("Display domain details")
 
     def get_parser(self, prog_name):
         parser = super(ShowDomain, self).get_parser(prog_name)
         parser.add_argument(
             'domain',
             metavar='<domain>',
-            help='Domain to display (name or ID)',
+            help=_('Domain to display (name or ID)'),
         )
         return parser
 
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
+
+        domain_str = common._get_token_resource(identity_client, 'domain',
+                                                parsed_args.domain)
+
         domain = utils.find_resource(identity_client.domains,
-                                     parsed_args.domain)
+                                     domain_str)
 
         domain._info.pop('links')
-        return zip(*sorted(six.iteritems(domain._info)))
+        return zip(*sorted(domain._info.items()))
