@@ -15,14 +15,19 @@
 
 """Identity v3 Endpoint action implementations"""
 
-import six
-import sys
+import logging
 
-from openstackclient.common import command
-from openstackclient.common import utils
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
+
+from openstackclient.i18n import _
 from openstackclient.identity import common
 
 
+LOG = logging.getLogger(__name__)
+
+
 def get_service_name(service):
     if hasattr(service, 'name'):
         return service.name
@@ -30,31 +35,67 @@ def get_service_name(service):
         return ''
 
 
+class AddProjectToEndpoint(command.Command):
+    _description = _("Associate a project to an endpoint")
+
+    def get_parser(self, prog_name):
+        parser = super(
+            AddProjectToEndpoint, self).get_parser(prog_name)
+        parser.add_argument(
+            'endpoint',
+            metavar='<endpoint>',
+            help=_('Endpoint to associate with '
+                   'specified project (name or ID)'),
+        )
+        parser.add_argument(
+            'project',
+            metavar='<project>',
+            help=_('Project to associate with '
+                   'specified endpoint name or ID)'),
+        )
+        common.add_project_domain_option_to_parser(parser)
+        return parser
+
+    def take_action(self, parsed_args):
+        client = self.app.client_manager.identity
+
+        endpoint = utils.find_resource(client.endpoints,
+                                       parsed_args.endpoint)
+
+        project = common.find_project(client,
+                                      parsed_args.project,
+                                      parsed_args.project_domain)
+
+        client.endpoint_filter.add_endpoint_to_project(
+            project=project.id,
+            endpoint=endpoint.id)
+
+
 class CreateEndpoint(command.ShowOne):
-    """Create new endpoint"""
+    _description = _("Create new endpoint")
 
     def get_parser(self, prog_name):
         parser = super(CreateEndpoint, self).get_parser(prog_name)
         parser.add_argument(
             'service',
             metavar='<service>',
-            help='New endpoint service (name or ID)',
+            help=_('Service to be associated with new endpoint (name or ID)'),
         )
         parser.add_argument(
             'interface',
             metavar='<interface>',
             choices=['admin', 'public', 'internal'],
-            help='New endpoint interface type (admin, public or internal)',
+            help=_('New endpoint interface type (admin, public or internal)'),
         )
         parser.add_argument(
             'url',
             metavar='<url>',
-            help='New endpoint URL',
+            help=_('New endpoint URL'),
         )
         parser.add_argument(
             '--region',
             metavar='<region-id>',
-            help='New endpoint region ID',
+            help=_('New endpoint region ID'),
         )
         enable_group = parser.add_mutually_exclusive_group()
         enable_group.add_argument(
@@ -62,13 +103,13 @@ class CreateEndpoint(command.ShowOne):
             dest='enabled',
             action='store_true',
             default=True,
-            help='Enable endpoint (default)',
+            help=_('Enable endpoint (default)'),
         )
         enable_group.add_argument(
             '--disable',
             dest='enabled',
             action='store_false',
-            help='Disable endpoint',
+            help=_('Disable endpoint'),
         )
         return parser
 
@@ -89,69 +130,125 @@ class CreateEndpoint(command.ShowOne):
         info.update(endpoint._info)
         info['service_name'] = get_service_name(service)
         info['service_type'] = service.type
-        return zip(*sorted(six.iteritems(info)))
+        return zip(*sorted(info.items()))
 
 
 class DeleteEndpoint(command.Command):
-    """Delete endpoint"""
+    _description = _("Delete endpoint(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteEndpoint, self).get_parser(prog_name)
         parser.add_argument(
             'endpoint',
             metavar='<endpoint-id>',
-            help='Endpoint ID to delete',
+            nargs='+',
+            help=_('Endpoint(s) to delete (ID only)'),
         )
         return parser
 
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
-        endpoint_id = utils.find_resource(identity_client.endpoints,
-                                          parsed_args.endpoint).id
-        identity_client.endpoints.delete(endpoint_id)
+        result = 0
+        for i in parsed_args.endpoint:
+            try:
+                endpoint_id = utils.find_resource(
+                    identity_client.endpoints, i).id
+                identity_client.endpoints.delete(endpoint_id)
+            except Exception as e:
+                result += 1
+                LOG.error(_("Failed to delete endpoint with "
+                          "ID '%(endpoint)s': %(e)s"), {'endpoint': i, 'e': e})
+
+        if result > 0:
+            total = len(parsed_args.endpoint)
+            msg = (_("%(result)s of %(total)s endpoints failed "
+                   "to delete.") % {'result': result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListEndpoint(command.Lister):
-    """List endpoints"""
+    _description = _("List endpoints")
 
     def get_parser(self, prog_name):
         parser = super(ListEndpoint, self).get_parser(prog_name)
         parser.add_argument(
             '--service',
             metavar='<service>',
-            help='Filter by service',
+            help=_('Filter by service (type, name or ID)'),
         )
         parser.add_argument(
             '--interface',
             metavar='<interface>',
             choices=['admin', 'public', 'internal'],
-            help='Filter by interface type (admin, public or internal)',
+            help=_('Filter by interface type (admin, public or internal)'),
         )
         parser.add_argument(
             '--region',
             metavar='<region-id>',
-            help='Filter by region ID',
+            help=_('Filter by region ID'),
         )
+        list_group = parser.add_mutually_exclusive_group()
+        list_group.add_argument(
+            '--endpoint',
+            metavar='<endpoint-group>',
+            help=_('Endpoint to list filters'),
+        )
+        list_group.add_argument(
+            '--project',
+            metavar='<project>',
+            help=_('Project to list filters (name or ID)'),
+        )
+        common.add_project_domain_option_to_parser(parser)
         return parser
 
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
-        columns = ('ID', 'Region', 'Service Name', 'Service Type',
-                   'Enabled', 'Interface', 'URL')
-        kwargs = {}
-        if parsed_args.service:
-            service = common.find_service(identity_client, parsed_args.service)
-            kwargs['service'] = service.id
-        if parsed_args.interface:
-            kwargs['interface'] = parsed_args.interface
-        if parsed_args.region:
-            kwargs['region'] = parsed_args.region
-        data = identity_client.endpoints.list(**kwargs)
 
-        for ep in data:
-            service = common.find_service(identity_client, ep.service_id)
-            ep.service_name = get_service_name(service)
-            ep.service_type = service.type
+        endpoint = None
+        if parsed_args.endpoint:
+            endpoint = utils.find_resource(identity_client.endpoints,
+                                           parsed_args.endpoint)
+        project = None
+        if parsed_args.project:
+            project = common.find_project(identity_client,
+                                          parsed_args.project,
+                                          parsed_args.project_domain)
+
+        if endpoint:
+            columns = ('ID', 'Name')
+            data = (
+                identity_client.endpoint_filter
+                .list_projects_for_endpoint(endpoint=endpoint.id)
+            )
+        else:
+            columns = ('ID', 'Region', 'Service Name', 'Service Type',
+                       'Enabled', 'Interface', 'URL')
+            kwargs = {}
+            if parsed_args.service:
+                service = common.find_service(identity_client,
+                                              parsed_args.service)
+                kwargs['service'] = service.id
+            if parsed_args.interface:
+                kwargs['interface'] = parsed_args.interface
+            if parsed_args.region:
+                kwargs['region'] = parsed_args.region
+
+            if project:
+                data = (
+                    identity_client.endpoint_filter
+                    .list_endpoints_for_project(project=project.id)
+                )
+            else:
+                data = identity_client.endpoints.list(**kwargs)
+
+            service_list = identity_client.services.list()
+
+            for ep in data:
+                service = common.find_service_in_list(service_list,
+                                                      ep.service_id)
+                ep.service_name = get_service_name(service)
+                ep.service_type = service.type
+
         return (columns,
                 (utils.get_item_properties(
                     s, columns,
@@ -159,49 +256,85 @@ class ListEndpoint(command.Lister):
                 ) for s in data))
 
 
+class RemoveProjectFromEndpoint(command.Command):
+    _description = _("Dissociate a project from an endpoint")
+
+    def get_parser(self, prog_name):
+        parser = super(
+            RemoveProjectFromEndpoint, self).get_parser(prog_name)
+        parser.add_argument(
+            'endpoint',
+            metavar='<endpoint>',
+            help=_('Endpoint to dissociate from '
+                   'specified project (name or ID)'),
+        )
+        parser.add_argument(
+            'project',
+            metavar='<project>',
+            help=_('Project to dissociate from '
+                   'specified endpoint name or ID)'),
+        )
+        common.add_project_domain_option_to_parser(parser)
+        return parser
+
+    def take_action(self, parsed_args):
+        client = self.app.client_manager.identity
+
+        endpoint = utils.find_resource(client.endpoints,
+                                       parsed_args.endpoint)
+
+        project = common.find_project(client,
+                                      parsed_args.project,
+                                      parsed_args.project_domain)
+
+        client.endpoint_filter.delete_endpoint_from_project(
+            project=project.id,
+            endpoint=endpoint.id)
+
+
 class SetEndpoint(command.Command):
-    """Set endpoint properties"""
+    _description = _("Set endpoint properties")
 
     def get_parser(self, prog_name):
         parser = super(SetEndpoint, self).get_parser(prog_name)
         parser.add_argument(
             'endpoint',
             metavar='<endpoint-id>',
-            help='Endpoint ID to modify',
+            help=_('Endpoint to modify (ID only)'),
         )
         parser.add_argument(
             '--region',
             metavar='<region-id>',
-            help='New endpoint region ID',
+            help=_('New endpoint region ID'),
         )
         parser.add_argument(
             '--interface',
             metavar='<interface>',
             choices=['admin', 'public', 'internal'],
-            help='New endpoint interface type (admin, public or internal)',
+            help=_('New endpoint interface type (admin, public or internal)'),
         )
         parser.add_argument(
             '--url',
             metavar='<url>',
-            help='New endpoint URL',
+            help=_('New endpoint URL'),
         )
         parser.add_argument(
             '--service',
             metavar='<service>',
-            help='New endpoint service (name or ID)',
+            help=_('New endpoint service (name or ID)'),
         )
         enable_group = parser.add_mutually_exclusive_group()
         enable_group.add_argument(
             '--enable',
             dest='enabled',
             action='store_true',
-            help='Enable endpoint',
+            help=_('Enable endpoint'),
         )
         enable_group.add_argument(
             '--disable',
             dest='disabled',
             action='store_true',
-            help='Disable endpoint',
+            help=_('Disable endpoint'),
         )
         return parser
 
@@ -210,12 +343,6 @@ class SetEndpoint(command.Command):
         endpoint = utils.find_resource(identity_client.endpoints,
                                        parsed_args.endpoint)
 
-        if (not parsed_args.interface and not parsed_args.url
-                and not parsed_args.service and not parsed_args.region
-                and not parsed_args.enabled and not parsed_args.disabled):
-            sys.stdout.write("Endpoint not updated, no arguments present")
-            return
-
         service_id = None
         if parsed_args.service:
             service = common.find_service(identity_client, parsed_args.service)
@@ -237,14 +364,15 @@ class SetEndpoint(command.Command):
 
 
 class ShowEndpoint(command.ShowOne):
-    """Display endpoint details"""
+    _description = _("Display endpoint details")
 
     def get_parser(self, prog_name):
         parser = super(ShowEndpoint, self).get_parser(prog_name)
         parser.add_argument(
             'endpoint',
-            metavar='<endpoint-id>',
-            help='Endpoint ID to display',
+            metavar='<endpoint>',
+            help=_('Endpoint to display (endpoint ID, service ID,'
+                   ' service name, service type)'),
         )
         return parser
 
@@ -260,4 +388,4 @@ class ShowEndpoint(command.ShowOne):
         info.update(endpoint._info)
         info['service_name'] = get_service_name(service)
         info['service_type'] = service.type
-        return zip(*sorted(six.iteritems(info)))
+        return zip(*sorted(info.items()))
