@@ -22,7 +22,6 @@ from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
-import six
 
 from openstackclient.i18n import _
 from openstackclient.identity import common
@@ -103,14 +102,6 @@ class CreateProject(command.ShowOne):
         kwargs = {}
         if parsed_args.property:
             kwargs = parsed_args.property.copy()
-        if 'is_domain' in kwargs.keys():
-            if kwargs['is_domain'].lower() == "true":
-                kwargs['is_domain'] = True
-            elif kwargs['is_domain'].lower() == "false":
-                kwargs['is_domain'] = False
-            elif kwargs['is_domain'].lower() == "none":
-                kwargs['is_domain'] = None
-
         kwargs['tags'] = list(set(parsed_args.tags))
 
         try:
@@ -132,7 +123,7 @@ class CreateProject(command.ShowOne):
                 raise
 
         project._info.pop('links')
-        return zip(*sorted(six.iteritems(project._info)))
+        return zip(*sorted(project._info.items()))
 
 
 class DeleteProject(command.Command):
@@ -193,6 +184,11 @@ class ListProject(command.Lister):
             metavar='<domain>',
             help=_('Filter projects by <domain> (name or ID)'),
         )
+        parser.add_argument(
+            '--parent',
+            metavar='<parent>',
+            help=_('Filter projects whose parent is <parent> (name or ID)'),
+        )
         parser.add_argument(
             '--user',
             metavar='<user>',
@@ -234,6 +230,11 @@ class ListProject(command.Lister):
                                            parsed_args.domain).id
             kwargs['domain'] = domain_id
 
+        if parsed_args.parent:
+            parent_id = common.find_project(identity_client,
+                                            parsed_args.parent).id
+            kwargs['parent'] = parent_id
+
         if parsed_args.user:
             if parsed_args.domain:
                 user_id = utils.find_resource(identity_client.users,
@@ -399,4 +400,4 @@ class ShowProject(command.ShowOne):
                 subtree_as_ids=parsed_args.children)
 
         project._info.pop('links')
-        return zip(*sorted(six.iteritems(project._info)))
+        return zip(*sorted(project._info.items()))
