@@ -15,6 +15,7 @@
 
 import logging
 
+from osc_lib.cli import format_columns
 from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
@@ -26,6 +27,11 @@ from openstackclient.network import sdk_utils
 LOG = logging.getLogger(__name__)
 
 
+_formatters = {
+    'location': format_columns.DictColumn,
+}
+
+
 def _get_columns(item):
     return sdk_utils.get_osc_show_columns_for_sdk_resource(item, {})
 
@@ -90,7 +96,7 @@ class CreateNetworkSegment(command.ShowOne):
             attrs['segmentation_id'] = parsed_args.segment
         obj = client.create_segment(**attrs)
         display_columns, columns = _get_columns(obj)
-        data = utils.get_item_properties(obj, columns)
+        data = utils.get_item_properties(obj, columns, formatters=_formatters)
         return (display_columns, data)
 
 
@@ -119,8 +125,8 @@ class DeleteNetworkSegment(command.Command):
             except Exception as e:
                 result += 1
                 LOG.error(_("Failed to delete network segment with "
-                            "ID '%(network_segment)s': %(e)s")
-                          % {'network_segment': network_segment, 'e': e})
+                            "ID '%(network_segment)s': %(e)s"),
+                          {'network_segment': network_segment, 'e': e})
 
         if result > 0:
             total = len(parsed_args.network_segment)
@@ -242,5 +248,5 @@ class ShowNetworkSegment(command.ShowOne):
             ignore_missing=False
         )
         display_columns, columns = _get_columns(obj)
-        data = utils.get_item_properties(obj, columns)
+        data = utils.get_item_properties(obj, columns, formatters=_formatters)
         return (display_columns, data)
