@@ -13,52 +13,57 @@
 
 """Router action implementations"""
 
-import argparse
 import copy
 import json
 import logging
 
+from cliff import columns as cliff_columns
+from osc_lib.cli import format_columns
 from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
+from osc_lib.utils import tags as _tag
 
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
 from openstackclient.network import sdk_utils
-from openstackclient.network.v2 import _tag
 
 
 LOG = logging.getLogger(__name__)
 
 
-def _format_admin_state(state):
-    return 'UP' if state else 'DOWN'
+class AdminStateColumn(cliff_columns.FormattableColumn):
+    def human_readable(self):
+        return 'UP' if self._value else 'DOWN'
 
 
-def _format_router_info(info):
-    try:
-        return json.dumps(info)
-    except (TypeError, KeyError):
-        return ''
+class RouterInfoColumn(cliff_columns.FormattableColumn):
+    def human_readable(self):
+        try:
+            return json.dumps(self._value)
+        except (TypeError, KeyError):
+            return ''
 
 
-def _format_routes(routes):
-    # Map the route keys to match --route option.
-    for route in routes:
-        if 'nexthop' in route:
-            route['gateway'] = route.pop('nexthop')
-    return utils.format_list_of_dicts(routes)
+class RoutesColumn(cliff_columns.FormattableColumn):
+    def human_readable(self):
+        # Map the route keys to match --route option.
+        for route in self._value or []:
+            if 'nexthop' in route:
+                route['gateway'] = route.pop('nexthop')
+        return utils.format_list_of_dicts(self._value)
 
 
 _formatters = {
-    'admin_state_up': _format_admin_state,
-    'is_admin_state_up': _format_admin_state,
-    'external_gateway_info': _format_router_info,
-    'availability_zones': utils.format_list,
-    'availability_zone_hints': utils.format_list,
-    'routes': _format_routes,
-    'tags': utils.format_list,
+    'admin_state_up': AdminStateColumn,
+    'is_admin_state_up': AdminStateColumn,
+    'external_gateway_info': RouterInfoColumn,
+    'availability_zones': format_columns.ListColumn,
+    'availability_zone_hints': format_columns.ListColumn,
+    'location': format_columns.DictColumn,
+    'routes': RoutesColumn,
+    'tags': format_columns.ListColumn,
 }
 
 
@@ -71,13 +76,21 @@ def _get_columns(item):
     }
     if hasattr(item, 'interfaces_info'):
         column_map['interfaces_info'] = 'interfaces_info'
-    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)
+    invisible_columns = []
+    if item.is_ha is None:
+        invisible_columns.append('is_ha')
+        column_map.pop('is_ha')
+    if item.is_distributed is None:
+        invisible_columns.append('is_distributed')
+        column_map.pop('is_distributed')
+    return sdk_utils.get_osc_show_columns_for_sdk_resource(
+        item, column_map, invisible_columns)
 
 
 def _get_attrs(client_manager, parsed_args):
     attrs = {}
     if parsed_args.name is not None:
-        attrs['name'] = str(parsed_args.name)
+        attrs['name'] = parsed_args.name
     if parsed_args.enable:
         attrs['admin_state_up'] = True
     if parsed_args.disable:
@@ -86,8 +99,8 @@ def _get_attrs(client_manager, parsed_args):
         attrs['distributed'] = False
     if parsed_args.distributed:
         attrs['distributed'] = True
-    if ('availability_zone_hints' in parsed_args
-            and parsed_args.availability_zone_hints is not None):
+    if ('availability_zone_hints' in parsed_args and
+            parsed_args.availability_zone_hints is not None):
         attrs['availability_zone_hints'] = parsed_args.availability_zone_hints
     if parsed_args.description is not None:
         attrs['description'] = parsed_args.description
@@ -330,8 +343,6 @@ class ListRouter(command.Lister):
             'name',
             'status',
             'is_admin_state_up',
-            'is_distributed',
-            'is_ha',
             'project_id',
         )
         column_headers = (
@@ -339,8 +350,6 @@ class ListRouter(command.Lister):
             'Name',
             'Status',
             'State',
-            'Distributed',
-            'HA',
             'Project',
         )
 
@@ -376,6 +385,16 @@ class ListRouter(command.Lister):
         else:
             data = client.routers(**args)
 
+        # check if "HA" and "Distributed" columns should be displayed also
+        data = list(data)
+        for d in data:
+            if (d.is_distributed is not None and
+                    'is_distributed' not in columns):
+                columns = columns + ('is_distributed',)
+                column_headers = column_headers + ('Distributed',)
+            if d.is_ha is not None and 'is_ha' not in columns:
+                columns = columns + ('is_ha',)
+                column_headers = column_headers + ('HA',)
         if parsed_args.long:
             columns = columns + (
                 'routes',
@@ -514,8 +533,6 @@ class SetRouter(command.Command):
             action='store_true',
             help=_("Set router to centralized mode (disabled router only)")
         )
-        routes_group = parser.add_mutually_exclusive_group()
-        # ToDo(Reedip):Remove mutual exclusiveness once clear-routes is removed
         parser.add_argument(
             '--route',
             metavar='destination=<subnet>,gateway=<ip-address>',
@@ -528,18 +545,13 @@ class SetRouter(command.Command):
                    "gateway: nexthop IP address "
                    "(repeat option to set multiple routes)")
         )
-        routes_group.add_argument(
+        parser.add_argument(
             '--no-route',
             action='store_true',
             help=_("Clear routes associated with the router. "
                    "Specify both --route and --no-route to overwrite "
                    "current value of route.")
         )
-        routes_group.add_argument(
-            '--clear-routes',
-            action='store_true',
-            help=argparse.SUPPRESS,
-        )
         routes_ha = parser.add_mutually_exclusive_group()
         routes_ha.add_argument(
             '--ha',
@@ -563,7 +575,7 @@ class SetRouter(command.Command):
             metavar='subnet=<subnet>,ip-address=<ip-address>',
             action=parseractions.MultiKeyValueAction,
             optional_keys=['subnet', 'ip-address'],
-            help=_("Desired IP and/or subnet (name or ID)"
+            help=_("Desired IP and/or subnet (name or ID) "
                    "on external gateway: "
                    "subnet=<subnet>,ip-address=<ip-address> "
                    "(repeat option to set multiple fixed IP addresses)")
@@ -579,6 +591,17 @@ class SetRouter(command.Command):
             action='store_true',
             help=_("Disable Source NAT on external gateway")
         )
+        qos_policy_group = parser.add_mutually_exclusive_group()
+        qos_policy_group.add_argument(
+            '--qos-policy',
+            metavar='<qos-policy>',
+            help=_("Attach QoS policy to router gateway IPs")
+        )
+        qos_policy_group.add_argument(
+            '--no-qos-policy',
+            action='store_true',
+            help=_("Remove QoS policy from router gateway IPs")
+        )
         _tag.add_tag_option_to_parser_for_set(parser, _('router'))
         return parser
 
@@ -594,27 +617,22 @@ class SetRouter(command.Command):
             attrs['ha'] = True
         elif parsed_args.no_ha:
             attrs['ha'] = False
-        if parsed_args.clear_routes:
-            LOG.warning(_(
-                'The --clear-routes option is deprecated, '
-                'please use --no-route instead.'
-            ))
 
         if parsed_args.routes is not None:
             for route in parsed_args.routes:
                 route['nexthop'] = route.pop('gateway')
             attrs['routes'] = parsed_args.routes
-            if not (parsed_args.no_route or parsed_args.clear_routes):
+            if not parsed_args.no_route:
                 # Map the route keys and append to the current routes.
                 # The REST API will handle route validation and duplicates.
                 attrs['routes'] += obj.routes
-        elif parsed_args.no_route or parsed_args.clear_routes:
+        elif parsed_args.no_route:
             attrs['routes'] = []
         if (parsed_args.disable_snat or parsed_args.enable_snat or
-           parsed_args.fixed_ip) and not parsed_args.external_gateway:
-                msg = (_("You must specify '--external-gateway' in order"
-                         "to update the SNAT or fixed-ip values"))
-                raise exceptions.CommandError(msg)
+                parsed_args.fixed_ip) and not parsed_args.external_gateway:
+            msg = (_("You must specify '--external-gateway' in order "
+                     "to update the SNAT or fixed-ip values"))
+            raise exceptions.CommandError(msg)
         if parsed_args.external_gateway:
             gateway_info = {}
             network = client.find_network(
@@ -638,6 +656,27 @@ class SetRouter(command.Command):
                     ips.append(ip_spec)
                 gateway_info['external_fixed_ips'] = ips
             attrs['external_gateway_info'] = gateway_info
+
+        if ((parsed_args.qos_policy or parsed_args.no_qos_policy) and
+                not parsed_args.external_gateway):
+            try:
+                original_net_id = obj.external_gateway_info['network_id']
+            except (KeyError, TypeError):
+                msg = (_("You must specify '--external-gateway' or the router "
+                         "must already have an external network in order to "
+                         "set router gateway IP QoS"))
+                raise exceptions.CommandError(msg)
+            else:
+                if not attrs.get('external_gateway_info'):
+                    attrs['external_gateway_info'] = {}
+                attrs['external_gateway_info']['network_id'] = original_net_id
+        if parsed_args.qos_policy:
+            check_qos_id = client.find_qos_policy(
+                parsed_args.qos_policy, ignore_missing=False).id
+            attrs['external_gateway_info']['qos_policy_id'] = check_qos_id
+
+        if 'no_qos_policy' in parsed_args and parsed_args.no_qos_policy:
+            attrs['external_gateway_info']['qos_policy_id'] = None
         if attrs:
             client.update_router(obj, **attrs)
         # tags is a subresource and it needs to be updated separately.
@@ -674,7 +713,7 @@ class ShowRouter(command.ShowOne):
 
         setattr(obj, 'interfaces_info', interfaces_info)
         display_columns, columns = _get_columns(obj)
-        _formatters['interfaces_info'] = _format_router_info
+        _formatters['interfaces_info'] = RouterInfoColumn
         data = utils.get_item_properties(obj, columns, formatters=_formatters)
 
         return (display_columns, data)
@@ -701,6 +740,12 @@ class UnsetRouter(command.Command):
             action='store_true',
             default=False,
             help=_("Remove external gateway information from the router"))
+        parser.add_argument(
+            '--qos-policy',
+            action='store_true',
+            default=False,
+            help=_("Remove QoS policy from router gateway IPs")
+        )
         parser.add_argument(
             'router',
             metavar="<router>",
@@ -713,6 +758,7 @@ class UnsetRouter(command.Command):
         client = self.app.client_manager.network
         obj = client.find_router(parsed_args.router, ignore_missing=False)
         tmp_routes = copy.deepcopy(obj.routes)
+        tmp_external_gateway_info = copy.deepcopy(obj.external_gateway_info)
         attrs = {}
         if parsed_args.routes:
             try:
@@ -723,6 +769,20 @@ class UnsetRouter(command.Command):
                 msg = (_("Router does not contain route %s") % route)
                 raise exceptions.CommandError(msg)
             attrs['routes'] = tmp_routes
+        if parsed_args.qos_policy:
+            try:
+                if (tmp_external_gateway_info['network_id'] and
+                        tmp_external_gateway_info['qos_policy_id']):
+                    pass
+            except (KeyError, TypeError):
+                msg = _("Router does not have external network or qos policy")
+                raise exceptions.CommandError(msg)
+            else:
+                attrs['external_gateway_info'] = {
+                    'network_id': tmp_external_gateway_info['network_id'],
+                    'qos_policy_id': None
+                }
+
         if parsed_args.external_gateway:
             attrs['external_gateway_info'] = {}
         if attrs:
