@@ -16,15 +16,17 @@
 import copy
 import logging
 
+from cliff import columns as cliff_columns
+from osc_lib.cli import format_columns
 from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
+from osc_lib.utils import tags as _tag
 
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
 from openstackclient.network import sdk_utils
-from openstackclient.network.v2 import _tag
 
 
 LOG = logging.getLogger(__name__)
@@ -40,23 +42,28 @@ def _update_arguments(obj_list, parsed_args_list, option):
             raise exceptions.CommandError(msg)
 
 
-def _format_allocation_pools(data):
-    pool_formatted = ['%s-%s' % (pool.get('start', ''), pool.get('end', ''))
-                      for pool in data]
-    return ','.join(pool_formatted)
+class AllocationPoolsColumn(cliff_columns.FormattableColumn):
+    def human_readable(self):
+        pool_formatted = ['%s-%s' % (pool.get('start', ''),
+                                     pool.get('end', ''))
+                          for pool in self._value]
+        return ','.join(pool_formatted)
 
 
-def _format_host_routes(data):
-    # Map the host route keys to match --host-route option.
-    return utils.format_list_of_dicts(convert_entries_to_gateway(data))
+class HostRoutesColumn(cliff_columns.FormattableColumn):
+    def human_readable(self):
+        # Map the host route keys to match --host-route option.
+        return utils.format_list_of_dicts(
+            convert_entries_to_gateway(self._value))
 
 
 _formatters = {
-    'allocation_pools': _format_allocation_pools,
-    'dns_nameservers': utils.format_list,
-    'host_routes': _format_host_routes,
-    'service_types': utils.format_list,
-    'tags': utils.format_list,
+    'allocation_pools': AllocationPoolsColumn,
+    'dns_nameservers': format_columns.ListColumn,
+    'host_routes': HostRoutesColumn,
+    'location': format_columns.DictColumn,
+    'service_types': format_columns.ListColumn,
+    'tags': format_columns.ListColumn,
 }
 
 
@@ -167,8 +174,9 @@ def convert_entries_to_gateway(entries):
 
 def _get_attrs(client_manager, parsed_args, is_create=True):
     attrs = {}
+    client = client_manager.network
     if 'name' in parsed_args and parsed_args.name is not None:
-        attrs['name'] = str(parsed_args.name)
+        attrs['name'] = parsed_args.name
 
     if is_create:
         if 'project' in parsed_args and parsed_args.project is not None:
@@ -179,7 +187,6 @@ def _get_attrs(client_manager, parsed_args, is_create=True):
                 parsed_args.project_domain,
             ).id
             attrs['tenant_id'] = project_id
-        client = client_manager.network
         attrs['network_id'] = client.find_network(parsed_args.network,
                                                   ignore_missing=False).id
         if parsed_args.subnet_pool is not None:
@@ -200,10 +207,10 @@ def _get_attrs(client_manager, parsed_args, is_create=True):
             attrs['ipv6_ra_mode'] = parsed_args.ipv6_ra_mode
         if parsed_args.ipv6_address_mode is not None:
             attrs['ipv6_address_mode'] = parsed_args.ipv6_address_mode
-        if parsed_args.network_segment is not None:
-            attrs['segment_id'] = client.find_segment(
-                parsed_args.network_segment, ignore_missing=False).id
 
+    if parsed_args.network_segment is not None:
+        attrs['segment_id'] = client.find_segment(
+            parsed_args.network_segment, ignore_missing=False).id
     if 'gateway' in parsed_args and parsed_args.gateway is not None:
         gateway = parsed_args.gateway.lower()
 
@@ -223,6 +230,10 @@ def _get_attrs(client_manager, parsed_args, is_create=True):
         attrs['enable_dhcp'] = True
     if parsed_args.no_dhcp:
         attrs['enable_dhcp'] = False
+    if parsed_args.dns_publish_fixed_ip:
+        attrs['dns_publish_fixed_ip'] = True
+    if parsed_args.no_dns_publish_fixed_ip:
+        attrs['dns_publish_fixed_ip'] = False
     if ('dns_nameservers' in parsed_args and
        parsed_args.dns_nameservers is not None):
         attrs['dns_nameservers'] = parsed_args.dns_nameservers
@@ -247,6 +258,7 @@ class CreateSubnet(command.ShowOne):
         parser = super(CreateSubnet, self).get_parser(prog_name)
         parser.add_argument(
             'name',
+            metavar='<name>',
             help=_("New subnet name")
         )
         parser.add_argument(
@@ -295,6 +307,17 @@ class CreateSubnet(command.ShowOne):
             action='store_true',
             help=_("Disable DHCP")
         )
+        dns_publish_fixed_ip_group = parser.add_mutually_exclusive_group()
+        dns_publish_fixed_ip_group.add_argument(
+            '--dns-publish-fixed-ip',
+            action='store_true',
+            help=_("Enable publishing fixed IPs in DNS")
+        )
+        dns_publish_fixed_ip_group.add_argument(
+            '--no-dns-publish-fixed-ip',
+            action='store_true',
+            help=_("Disable publishing fixed IPs in DNS (default)")
+        )
         parser.add_argument(
             '--gateway',
             metavar='<gateway>',
@@ -412,7 +435,7 @@ class ListSubnet(command.Lister):
             choices=[4, 6],
             metavar='<ip-version>',
             dest='ip_version',
-            help=_("List only subnets of given IP version in output."
+            help=_("List only subnets of given IP version in output. "
                    "Allowed values for IP version are 4 and 6."),
         )
         dhcp_enable_group = parser.add_mutually_exclusive_group()
@@ -550,6 +573,17 @@ class SetSubnet(command.Command):
             action='store_true',
             help=_("Disable DHCP")
         )
+        dns_publish_fixed_ip_group = parser.add_mutually_exclusive_group()
+        dns_publish_fixed_ip_group.add_argument(
+            '--dns-publish-fixed-ip',
+            action='store_true',
+            help=_("Enable publishing fixed IPs in DNS")
+        )
+        dns_publish_fixed_ip_group.add_argument(
+            '--no-dns-publish-fixed-ip',
+            action='store_true',
+            help=_("Disable publishing fixed IPs in DNS")
+        )
         parser.add_argument(
             '--gateway',
             metavar='<gateway>',
@@ -558,6 +592,14 @@ class SetSubnet(command.Command):
                    "'none': This subnet will not use a gateway, "
                    "e.g.: --gateway 192.168.9.1, --gateway none.")
         )
+        parser.add_argument(
+            '--network-segment',
+            metavar='<network-segment>',
+            help=_("Network segment to associate with this subnet (name or "
+                   "ID). It is only allowed to set the segment if the current "
+                   "value is `None`, the network must also have only one "
+                   "segment and only one subnet can exist on the network.")
+        )
         parser.add_argument(
             '--description',
             metavar='<description>',
@@ -581,7 +623,7 @@ class SetSubnet(command.Command):
             if not parsed_args.no_host_route:
                 attrs['host_routes'] += obj.host_routes
         elif parsed_args.no_host_route:
-            attrs['host_routes'] = ''
+            attrs['host_routes'] = []
         if 'allocation_pools' in attrs:
             if not parsed_args.no_allocation_pool:
                 attrs['allocation_pools'] += obj.allocation_pools
@@ -672,29 +714,30 @@ class UnsetSubnet(command.Command):
     def take_action(self, parsed_args):
         client = self.app.client_manager.network
         obj = client.find_subnet(parsed_args.subnet, ignore_missing=False)
-        tmp_obj = copy.deepcopy(obj)
+
         attrs = {}
         if parsed_args.dns_nameservers:
-            _update_arguments(tmp_obj.dns_nameservers,
+            attrs['dns_nameservers'] = copy.deepcopy(obj.dns_nameservers)
+            _update_arguments(attrs['dns_nameservers'],
                               parsed_args.dns_nameservers,
                               'dns-nameserver')
-            attrs['dns_nameservers'] = tmp_obj.dns_nameservers
         if parsed_args.host_routes:
+            attrs['host_routes'] = copy.deepcopy(obj.host_routes)
             _update_arguments(
-                tmp_obj.host_routes,
+                attrs['host_routes'],
                 convert_entries_to_nexthop(parsed_args.host_routes),
                 'host-route')
-            attrs['host_routes'] = tmp_obj.host_routes
         if parsed_args.allocation_pools:
-            _update_arguments(tmp_obj.allocation_pools,
+            attrs['allocation_pools'] = copy.deepcopy(obj.allocation_pools)
+            _update_arguments(attrs['allocation_pools'],
                               parsed_args.allocation_pools,
                               'allocation-pool')
-            attrs['allocation_pools'] = tmp_obj.allocation_pools
+
         if parsed_args.service_types:
-            _update_arguments(tmp_obj.service_types,
+            attrs['service_types'] = copy.deepcopy(obj.service_types)
+            _update_arguments(attrs['service_types'],
                               parsed_args.service_types,
                               'service-type')
-            attrs['service_types'] = tmp_obj.service_types
         if attrs:
             client.update_subnet(obj, **attrs)
 
