@@ -63,6 +63,127 @@ class ServerTests(common.ComputeTestCase):
         self.assertNotIn(name1, col_name)
         self.assertIn(name2, col_name)
 
+    def test_server_list_with_marker_and_deleted(self):
+        """Test server list with deleted and marker"""
+        cmd_output = self.server_create(cleanup=False)
+        name1 = cmd_output['name']
+        cmd_output = self.server_create(cleanup=False)
+        name2 = cmd_output['name']
+        id2 = cmd_output['id']
+        self.wait_for_status(name1, "ACTIVE")
+        self.wait_for_status(name2, "ACTIVE")
+
+        # Test list --marker with ID
+        cmd_output = json.loads(self.openstack(
+            'server list -f json --marker ' + id2
+        ))
+        col_name = [x["Name"] for x in cmd_output]
+        self.assertIn(name1, col_name)
+
+        # Test list --marker with Name
+        cmd_output = json.loads(self.openstack(
+            'server list -f json --marker ' + name2
+        ))
+        col_name = [x["Name"] for x in cmd_output]
+        self.assertIn(name1, col_name)
+
+        self.openstack('server delete --wait ' + name1)
+        self.openstack('server delete --wait ' + name2)
+
+        # Test list --deleted --marker with ID
+        cmd_output = json.loads(self.openstack(
+            'server list -f json --deleted --marker ' + id2
+        ))
+        col_name = [x["Name"] for x in cmd_output]
+        self.assertIn(name1, col_name)
+
+        # Test list --deleted --marker with Name
+        try:
+            cmd_output = json.loads(self.openstack(
+                'server list -f json --deleted --marker ' + name2
+            ))
+        except exceptions.CommandFailed as e:
+            self.assertIn('marker [%s] not found (HTTP 400)' % (name2),
+                          e.stderr.decode('utf-8'))
+
+    def test_server_list_with_changes_before(self):
+        """Test server list.
+
+        Getting the servers list with updated_at time equal or
+        before than changes-before.
+        """
+        cmd_output = self.server_create()
+        server_name1 = cmd_output['name']
+
+        cmd_output = self.server_create()
+        server_name2 = cmd_output['name']
+        updated_at2 = cmd_output['updated']
+
+        cmd_output = self.server_create()
+        server_name3 = cmd_output['name']
+
+        cmd_output = json.loads(self.openstack(
+            '--os-compute-api-version 2.66 ' +
+            'server list -f json '
+            '--changes-before ' + updated_at2
+        ))
+
+        col_updated = [server["Name"] for server in cmd_output]
+        self.assertIn(server_name1, col_updated)
+        self.assertIn(server_name2, col_updated)
+        self.assertNotIn(server_name3, col_updated)
+
+    def test_server_list_with_changes_since(self):
+        """Test server list.
+
+        Getting the servers list with updated_at time equal or
+        later than changes-since.
+        """
+        cmd_output = self.server_create()
+        server_name1 = cmd_output['name']
+        cmd_output = self.server_create()
+        server_name2 = cmd_output['name']
+        updated_at2 = cmd_output['updated']
+        cmd_output = self.server_create()
+        server_name3 = cmd_output['name']
+
+        cmd_output = json.loads(self.openstack(
+            'server list -f json '
+            '--changes-since ' + updated_at2
+        ))
+
+        col_updated = [server["Name"] for server in cmd_output]
+        self.assertNotIn(server_name1, col_updated)
+        self.assertIn(server_name2, col_updated)
+        self.assertIn(server_name3, col_updated)
+
+    def test_server_list_with_changes_before_and_changes_since(self):
+        """Test server list.
+
+        Getting the servers list with updated_at time equal or before than
+        changes-before and equal or later than changes-since.
+        """
+        cmd_output = self.server_create()
+        server_name1 = cmd_output['name']
+        cmd_output = self.server_create()
+        server_name2 = cmd_output['name']
+        updated_at2 = cmd_output['updated']
+        cmd_output = self.server_create()
+        server_name3 = cmd_output['name']
+        updated_at3 = cmd_output['updated']
+
+        cmd_output = json.loads(self.openstack(
+            '--os-compute-api-version 2.66 ' +
+            'server list -f json ' +
+            '--changes-since ' + updated_at2 +
+            ' --changes-before ' + updated_at3
+        ))
+
+        col_updated = [server["Name"] for server in cmd_output]
+        self.assertNotIn(server_name1, col_updated)
+        self.assertIn(server_name2, col_updated)
+        self.assertIn(server_name3, col_updated)
+
     def test_server_set(self):
         """Test server create, delete, set, show"""
         cmd_output = self.server_create()
@@ -407,7 +528,7 @@ class ServerTests(common.ComputeTestCase):
             cmd_output['status'],
         )
 
-        # NOTE(dtroyer): Prior to https://review.openstack.org/#/c/407111
+        # NOTE(dtroyer): Prior to https://review.opendev.org/#/c/407111
         #                --block-device-mapping was ignored if --volume
         #                present on the command line.  Now we should see the
         #                attachment.
@@ -536,6 +657,169 @@ class ServerTests(common.ComputeTestCase):
             # the attached volume had been deleted
             pass
 
+    def test_server_boot_with_bdm_image(self):
+        # Tests creating a server where the root disk is backed by the given
+        # --image but a --block-device-mapping with type=image is provided so
+        # that the compute service creates a volume from that image and
+        # attaches it as a non-root volume on the server. The block device is
+        # marked as delete_on_termination=True so it will be automatically
+        # deleted when the server is deleted.
+
+        # create server with bdm type=image
+        # NOTE(mriedem): This test is a bit unrealistic in that specifying the
+        # same image in the block device as the --image option does not really
+        # make sense, but we just want to make sure everything is processed
+        # as expected where nova creates a volume from the image and attaches
+        # that volume to the server.
+        server_name = uuid.uuid4().hex
+        server = json.loads(self.openstack(
+            'server create -f json ' +
+            '--flavor ' + self.flavor_name + ' ' +
+            '--image ' + self.image_name + ' ' +
+            '--block-device-mapping '
+            # This means create a 1GB volume from the specified image, attach
+            # it to the server at /dev/vdb and delete the volume when the
+            # server is deleted.
+            'vdb=' + self.image_name + ':image:1:true ' +
+            self.network_arg + ' ' +
+            '--wait ' +
+            server_name
+        ))
+        self.assertIsNotNone(server["id"])
+        self.assertEqual(
+            server_name,
+            server['name'],
+        )
+        self.wait_for_status(server_name, 'ACTIVE')
+
+        # check server volumes_attached, format is
+        # {"volumes_attached": "id='2518bc76-bf0b-476e-ad6b-571973745bb5'",}
+        cmd_output = json.loads(self.openstack(
+            'server show -f json ' +
+            server_name
+        ))
+        volumes_attached = cmd_output['volumes_attached']
+        self.assertTrue(volumes_attached.startswith('id='))
+        attached_volume_id = volumes_attached.replace('id=', '')
+
+        # check the volume that attached on server
+        cmd_output = json.loads(self.openstack(
+            'volume show -f json ' +
+            attached_volume_id
+        ))
+        attachments = cmd_output['attachments']
+        self.assertEqual(
+            1,
+            len(attachments),
+        )
+        self.assertEqual(
+            server['id'],
+            attachments[0]['server_id'],
+        )
+        self.assertEqual(
+            "in-use",
+            cmd_output['status'],
+        )
+        # TODO(mriedem): If we can parse the volume_image_metadata field from
+        # the volume show output we could assert the image_name is what we
+        # specified. volume_image_metadata is something like this:
+        # {u'container_format': u'bare', u'min_ram': u'0',
+        # u'disk_format': u'qcow2', u'image_name': u'cirros-0.4.0-x86_64-disk',
+        # u'image_id': u'05496c83-e2df-4c2f-9e48-453b6e49160d',
+        # u'checksum': u'443b7623e27ecf03dc9e01ee93f67afe', u'min_disk': u'0',
+        # u'size': u'12716032'}
+
+        # delete server, then check the attached volume has been deleted
+        self.openstack('server delete --wait ' + server_name)
+        cmd_output = json.loads(self.openstack(
+            'volume list -f json'
+        ))
+        target_volume = [each_volume
+                         for each_volume in cmd_output
+                         if each_volume['ID'] == attached_volume_id]
+        if target_volume:
+            # check the attached volume is 'deleting' status
+            self.assertEqual('deleting', target_volume[0]['Status'])
+        else:
+            # the attached volume had been deleted
+            pass
+
+    def test_boot_from_volume(self):
+        # Tests creating a server using --image and --boot-from-volume where
+        # the compute service will create a root volume of the specified size
+        # using the provided image, attach it as the root disk for the server
+        # and not delete the volume when the server is deleted.
+        server_name = uuid.uuid4().hex
+        server = json.loads(self.openstack(
+            'server create -f json ' +
+            '--flavor ' + self.flavor_name + ' ' +
+            '--image ' + self.image_name + ' ' +
+            '--boot-from-volume 1 ' +  # create a 1GB volume from the image
+            self.network_arg + ' ' +
+            '--wait ' +
+            server_name
+        ))
+        self.assertIsNotNone(server["id"])
+        self.assertEqual(
+            server_name,
+            server['name'],
+        )
+        self.wait_for_status(server_name, 'ACTIVE')
+
+        # check server volumes_attached, format is
+        # {"volumes_attached": "id='2518bc76-bf0b-476e-ad6b-571973745bb5'",}
+        cmd_output = json.loads(self.openstack(
+            'server show -f json ' +
+            server_name
+        ))
+        volumes_attached = cmd_output['volumes_attached']
+        self.assertTrue(volumes_attached.startswith('id='))
+        attached_volume_id = volumes_attached.replace('id=', '')
+        # Don't leak the volume when the test exits.
+        self.addCleanup(self.openstack, 'volume delete ' + attached_volume_id)
+
+        # Since the server is volume-backed the GET /servers/{server_id}
+        # response will have image=''.
+        self.assertEqual('', cmd_output['image'])
+
+        # check the volume that attached on server
+        cmd_output = json.loads(self.openstack(
+            'volume show -f json ' +
+            attached_volume_id
+        ))
+        # The volume size should be what we specified on the command line.
+        self.assertEqual(1, int(cmd_output['size']))
+        attachments = cmd_output['attachments']
+        self.assertEqual(
+            1,
+            len(attachments),
+        )
+        self.assertEqual(
+            server['id'],
+            attachments[0]['server_id'],
+        )
+        self.assertEqual(
+            "in-use",
+            cmd_output['status'],
+        )
+        # TODO(mriedem): If we can parse the volume_image_metadata field from
+        # the volume show output we could assert the image_name is what we
+        # specified. volume_image_metadata is something like this:
+        # {u'container_format': u'bare', u'min_ram': u'0',
+        # u'disk_format': u'qcow2', u'image_name': u'cirros-0.4.0-x86_64-disk',
+        # u'image_id': u'05496c83-e2df-4c2f-9e48-453b6e49160d',
+        # u'checksum': u'443b7623e27ecf03dc9e01ee93f67afe', u'min_disk': u'0',
+        # u'size': u'12716032'}
+
+        # delete server, then check the attached volume was not deleted
+        self.openstack('server delete --wait ' + server_name)
+        cmd_output = json.loads(self.openstack(
+            'volume show -f json ' +
+            attached_volume_id
+        ))
+        # check the volume is in 'available' status
+        self.assertEqual('available', cmd_output['status'])
+
     def test_server_create_with_none_network(self):
         """Test server create with none network option."""
         server_name = uuid.uuid4().hex
