@@ -14,12 +14,11 @@
 #
 
 import copy
+from unittest import mock
 
-import mock
 from osc_lib import exceptions
 from osc_lib import utils
 
-from openstackclient.identity import common
 from openstackclient.identity.v3 import role
 from openstackclient.tests.unit import fakes
 from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes
@@ -103,40 +102,6 @@ class TestRoleAdd(TestRole):
         # Get the command object to test
         self.cmd = role.AddRole(self.app, None)
 
-    def test_role_add_user_system(self):
-        arglist = [
-            '--user', identity_fakes.user_name,
-            '--system', 'all',
-            identity_fakes.role_name,
-        ]
-        if self._is_inheritance_testcase():
-            arglist.append('--inherited')
-        verifylist = [
-            ('user', identity_fakes.user_name),
-            ('group', None),
-            ('system', 'all'),
-            ('domain', None),
-            ('project', None),
-            ('role', identity_fakes.role_name),
-            ('inherited', self._is_inheritance_testcase()),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        result = self.cmd.take_action(parsed_args)
-
-        # Set expected values
-        kwargs = {
-            'user': identity_fakes.user_id,
-            'system': 'all',
-            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
-        }
-        # RoleManager.grant(role, user=, group=, domain=, project=)
-        self.roles_mock.grant.assert_called_with(
-            identity_fakes.role_id,
-            **kwargs
-        )
-        self.assertIsNone(result)
-
     def test_role_add_user_domain(self):
         arglist = [
             '--user', identity_fakes.user_name,
@@ -203,40 +168,6 @@ class TestRoleAdd(TestRole):
         )
         self.assertIsNone(result)
 
-    def test_role_add_group_system(self):
-        arglist = [
-            '--group', identity_fakes.group_name,
-            '--system', 'all',
-            identity_fakes.role_name,
-        ]
-        if self._is_inheritance_testcase():
-            arglist.append('--inherited')
-        verifylist = [
-            ('user', None),
-            ('group', identity_fakes.group_name),
-            ('system', 'all'),
-            ('domain', None),
-            ('project', None),
-            ('role', identity_fakes.role_name),
-            ('inherited', self._is_inheritance_testcase()),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        result = self.cmd.take_action(parsed_args)
-
-        # Set expected values
-        kwargs = {
-            'group': identity_fakes.group_id,
-            'system': 'all',
-            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
-        }
-        # RoleManager.grant(role, user=, group=, domain=, project=)
-        self.roles_mock.grant.assert_called_with(
-            identity_fakes.role_id,
-            **kwargs
-        )
-        self.assertIsNone(result)
-
     def test_role_add_group_domain(self):
         arglist = [
             '--group', identity_fakes.group_name,
@@ -401,6 +332,7 @@ class TestRoleCreate(TestRole):
         kwargs = {
             'domain': None,
             'name': identity_fakes.role_name,
+            'description': None,
         }
 
         # RoleManager.create(name=, domain=)
@@ -444,6 +376,7 @@ class TestRoleCreate(TestRole):
         kwargs = {
             'domain': identity_fakes.domain_id,
             'name': identity_fakes.ROLE_2['name'],
+            'description': None,
         }
 
         # RoleManager.create(name=, domain=)
@@ -460,6 +393,49 @@ class TestRoleCreate(TestRole):
         )
         self.assertEqual(datalist, data)
 
+    def test_role_create_with_description(self):
+
+        self.roles_mock.create.return_value = fakes.FakeResource(
+            None,
+            copy.deepcopy(identity_fakes.ROLE_2),
+            loaded=True,
+        )
+        arglist = [
+            '--description', identity_fakes.role_description,
+            identity_fakes.ROLE_2['name'],
+        ]
+        verifylist = [
+            ('description', identity_fakes.role_description),
+            ('name', identity_fakes.ROLE_2['name']),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
+        columns, data = self.cmd.take_action(parsed_args)
+
+        # Set expected values
+        kwargs = {
+            'description': identity_fakes.role_description,
+            'name': identity_fakes.ROLE_2['name'],
+            'domain': None,
+        }
+
+        # RoleManager.create(name=, domain=)
+        self.roles_mock.create.assert_called_with(
+            **kwargs
+        )
+
+        collist = ('domain', 'id', 'name')
+        self.assertEqual(collist, columns)
+        datalist = (
+            'd1',
+            identity_fakes.ROLE_2['id'],
+            identity_fakes.ROLE_2['name'],
+        )
+        self.assertEqual(datalist, data)
+
 
 class TestRoleDelete(TestRole):
 
@@ -675,81 +651,6 @@ class TestRoleRemove(TestRole):
         # Get the command object to test
         self.cmd = role.RemoveRole(self.app, None)
 
-    def test_role_remove_user_system(self):
-        arglist = [
-            '--user', identity_fakes.user_name,
-            '--system', 'all',
-            identity_fakes.role_name
-        ]
-        if self._is_inheritance_testcase():
-            arglist.append('--inherited')
-        verifylist = [
-            ('user', identity_fakes.user_name),
-            ('group', None),
-            ('system', 'all'),
-            ('domain', None),
-            ('project', None),
-            ('role', identity_fakes.role_name),
-            ('inherited', self._is_inheritance_testcase()),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        result = self.cmd.take_action(parsed_args)
-
-        # Set expected values
-        kwargs = {
-            'user': identity_fakes.user_id,
-            'system': 'all',
-            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
-        }
-        # RoleManager.revoke(role, user=, group=, domain=, project=)
-        self.roles_mock.revoke.assert_called_with(
-            identity_fakes.role_id,
-            **kwargs
-        )
-        self.assertIsNone(result)
-
-    @mock.patch.object(common, 'find_user')
-    def test_role_remove_non_existent_user_system(self, find_mock):
-        # Simulate the user not being in keystone, the client should gracefully
-        # handle this exception and send the request to remove the role since
-        # keystone supports removing role assignments with non-existent actors
-        # (e.g., users or groups).
-        find_mock.side_effect = exceptions.CommandError
-
-        arglist = [
-            '--user', identity_fakes.user_id,
-            '--system', 'all',
-            identity_fakes.role_name
-        ]
-        if self._is_inheritance_testcase():
-            arglist.append('--inherited')
-        verifylist = [
-            ('user', identity_fakes.user_id),
-            ('group', None),
-            ('system', 'all'),
-            ('domain', None),
-            ('project', None),
-            ('role', identity_fakes.role_name),
-            ('inherited', self._is_inheritance_testcase()),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        result = self.cmd.take_action(parsed_args)
-
-        # Set expected values
-        kwargs = {
-            'user': identity_fakes.user_id,
-            'system': 'all',
-            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
-        }
-        # RoleManager.revoke(role, user=, group=, domain=, project=)
-        self.roles_mock.revoke.assert_called_with(
-            identity_fakes.role_id,
-            **kwargs
-        )
-        self.assertIsNone(result)
-
     def test_role_remove_user_domain(self):
         arglist = [
             '--user', identity_fakes.user_name,
@@ -783,46 +684,6 @@ class TestRoleRemove(TestRole):
         )
         self.assertIsNone(result)
 
-    @mock.patch.object(common, 'find_user')
-    def test_role_remove_non_existent_user_domain(self, find_mock):
-        # Simulate the user not being in keystone, the client the gracefully
-        # handle this exception and send the request to remove the role since
-        # keystone will validate.
-        find_mock.side_effect = exceptions.CommandError
-
-        arglist = [
-            '--user', identity_fakes.user_id,
-            '--domain', identity_fakes.domain_name,
-            identity_fakes.role_name
-        ]
-        if self._is_inheritance_testcase():
-            arglist.append('--inherited')
-        verifylist = [
-            ('user', identity_fakes.user_id),
-            ('group', None),
-            ('system', None),
-            ('domain', identity_fakes.domain_name),
-            ('project', None),
-            ('role', identity_fakes.role_name),
-            ('inherited', self._is_inheritance_testcase()),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        result = self.cmd.take_action(parsed_args)
-
-        # Set expected values
-        kwargs = {
-            'user': identity_fakes.user_id,
-            'domain': identity_fakes.domain_id,
-            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
-        }
-        # RoleManager.revoke(role, user=, group=, domain=, project=)
-        self.roles_mock.revoke.assert_called_with(
-            identity_fakes.role_id,
-            **kwargs
-        )
-        self.assertIsNone(result)
-
     def test_role_remove_user_project(self):
         arglist = [
             '--user', identity_fakes.user_name,
@@ -856,121 +717,6 @@ class TestRoleRemove(TestRole):
         )
         self.assertIsNone(result)
 
-    @mock.patch.object(common, 'find_user')
-    def test_role_remove_non_existent_user_project(self, find_mock):
-        # Simulate the user not being in keystone, the client the gracefully
-        # handle this exception and send the request to remove the role since
-        # keystone will validate.
-        find_mock.side_effect = exceptions.CommandError
-
-        arglist = [
-            '--user', identity_fakes.user_id,
-            '--project', identity_fakes.project_name,
-            identity_fakes.role_name
-        ]
-        if self._is_inheritance_testcase():
-            arglist.append('--inherited')
-        verifylist = [
-            ('user', identity_fakes.user_id),
-            ('group', None),
-            ('system', None),
-            ('domain', None),
-            ('project', identity_fakes.project_name),
-            ('role', identity_fakes.role_name),
-            ('inherited', self._is_inheritance_testcase()),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        result = self.cmd.take_action(parsed_args)
-
-        # Set expected values
-        kwargs = {
-            'user': identity_fakes.user_id,
-            'project': identity_fakes.project_id,
-            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
-        }
-        # RoleManager.revoke(role, user=, group=, domain=, project=)
-        self.roles_mock.revoke.assert_called_with(
-            identity_fakes.role_id,
-            **kwargs
-        )
-        self.assertIsNone(result)
-
-    def test_role_remove_group_system(self):
-        arglist = [
-            '--group', identity_fakes.group_name,
-            '--system', 'all',
-            identity_fakes.role_name,
-        ]
-        if self._is_inheritance_testcase():
-            arglist.append('--inherited')
-        verifylist = [
-            ('user', None),
-            ('group', identity_fakes.group_name),
-            ('system', 'all'),
-            ('domain', None),
-            ('project', None),
-            ('role', identity_fakes.role_name),
-            ('role', identity_fakes.role_name),
-            ('inherited', self._is_inheritance_testcase()),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        result = self.cmd.take_action(parsed_args)
-
-        # Set expected values
-        kwargs = {
-            'group': identity_fakes.group_id,
-            'system': 'all',
-            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
-        }
-        # RoleManager.revoke(role, user=, group=, domain=, project=)
-        self.roles_mock.revoke.assert_called_with(
-            identity_fakes.role_id,
-            **kwargs
-        )
-        self.assertIsNone(result)
-
-    @mock.patch.object(common, 'find_group')
-    def test_role_remove_non_existent_group_system(self, find_mock):
-        # Simulate the user not being in keystone, the client the gracefully
-        # handle this exception and send the request to remove the role since
-        # keystone will validate.
-        find_mock.side_effect = exceptions.CommandError
-
-        arglist = [
-            '--group', identity_fakes.group_id,
-            '--system', 'all',
-            identity_fakes.role_name
-        ]
-        if self._is_inheritance_testcase():
-            arglist.append('--inherited')
-        verifylist = [
-            ('user', None),
-            ('group', identity_fakes.group_id),
-            ('system', 'all'),
-            ('domain', None),
-            ('project', None),
-            ('role', identity_fakes.role_name),
-            ('inherited', self._is_inheritance_testcase()),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        result = self.cmd.take_action(parsed_args)
-
-        # Set expected values
-        kwargs = {
-            'group': identity_fakes.group_id,
-            'system': 'all',
-            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
-        }
-        # RoleManager.revoke(role, user=, group=, domain=, project=)
-        self.roles_mock.revoke.assert_called_with(
-            identity_fakes.role_id,
-            **kwargs
-        )
-        self.assertIsNone(result)
-
     def test_role_remove_group_domain(self):
         arglist = [
             '--group', identity_fakes.group_name,
@@ -1005,46 +751,6 @@ class TestRoleRemove(TestRole):
         )
         self.assertIsNone(result)
 
-    @mock.patch.object(common, 'find_group')
-    def test_role_remove_non_existent_group_domain(self, find_mock):
-        # Simulate the user not being in keystone, the client the gracefully
-        # handle this exception and send the request to remove the role since
-        # keystone will validate.
-        find_mock.side_effect = exceptions.CommandError
-
-        arglist = [
-            '--group', identity_fakes.group_id,
-            '--domain', identity_fakes.domain_name,
-            identity_fakes.role_name
-        ]
-        if self._is_inheritance_testcase():
-            arglist.append('--inherited')
-        verifylist = [
-            ('user', None),
-            ('group', identity_fakes.group_id),
-            ('system', None),
-            ('domain', identity_fakes.domain_name),
-            ('project', None),
-            ('role', identity_fakes.role_name),
-            ('inherited', self._is_inheritance_testcase()),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        result = self.cmd.take_action(parsed_args)
-
-        # Set expected values
-        kwargs = {
-            'group': identity_fakes.group_id,
-            'domain': identity_fakes.domain_id,
-            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
-        }
-        # RoleManager.revoke(role, user=, group=, domain=, project=)
-        self.roles_mock.revoke.assert_called_with(
-            identity_fakes.role_id,
-            **kwargs
-        )
-        self.assertIsNone(result)
-
     def test_role_remove_group_project(self):
         arglist = [
             '--group', identity_fakes.group_name,
@@ -1078,46 +784,6 @@ class TestRoleRemove(TestRole):
         )
         self.assertIsNone(result)
 
-    @mock.patch.object(common, 'find_group')
-    def test_role_remove_non_existent_group_project(self, find_mock):
-        # Simulate the user not being in keystone, the client the gracefully
-        # handle this exception and send the request to remove the role since
-        # keystone will validate.
-        find_mock.side_effect = exceptions.CommandError
-
-        arglist = [
-            '--group', identity_fakes.group_id,
-            '--project', identity_fakes.project_name,
-            identity_fakes.role_name
-        ]
-        if self._is_inheritance_testcase():
-            arglist.append('--inherited')
-        verifylist = [
-            ('user', None),
-            ('group', identity_fakes.group_id),
-            ('system', None),
-            ('domain', None),
-            ('project', identity_fakes.project_name),
-            ('role', identity_fakes.role_name),
-            ('inherited', self._is_inheritance_testcase()),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        result = self.cmd.take_action(parsed_args)
-
-        # Set expected values
-        kwargs = {
-            'group': identity_fakes.group_id,
-            'project': identity_fakes.project_id,
-            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
-        }
-        # RoleManager.revoke(role, user=, group=, domain=, project=)
-        self.roles_mock.revoke.assert_called_with(
-            identity_fakes.role_id,
-            **kwargs
-        )
-        self.assertIsNone(result)
-
     def test_role_remove_domain_role_on_group_domain(self):
         self.roles_mock.get.return_value = fakes.FakeResource(
             None,
@@ -1204,6 +870,7 @@ class TestRoleSet(TestRole):
         # Set expected values
         kwargs = {
             'name': 'over',
+            'description': None,
         }
         # RoleManager.update(role, name=)
         self.roles_mock.update.assert_called_with(
@@ -1235,6 +902,39 @@ class TestRoleSet(TestRole):
         # Set expected values
         kwargs = {
             'name': 'over',
+            'description': None,
+        }
+        # RoleManager.update(role, name=)
+        self.roles_mock.update.assert_called_with(
+            identity_fakes.ROLE_2['id'],
+            **kwargs
+        )
+        self.assertIsNone(result)
+
+    def test_role_set_description(self):
+        self.roles_mock.get.return_value = fakes.FakeResource(
+            None,
+            copy.deepcopy(identity_fakes.ROLE_2),
+            loaded=True,
+        )
+        arglist = [
+            '--name', 'over',
+            '--description', identity_fakes.role_description,
+            identity_fakes.ROLE_2['name'],
+        ]
+        verifylist = [
+            ('name', 'over'),
+            ('description', identity_fakes.role_description),
+            ('role', identity_fakes.ROLE_2['name']),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+
+        # Set expected values
+        kwargs = {
+            'name': 'over',
+            'description': identity_fakes.role_description,
         }
         # RoleManager.update(role, name=)
         self.roles_mock.update.assert_called_with(
