@@ -14,8 +14,8 @@
 #
 
 import contextlib
+from unittest import mock
 
-import mock
 from osc_lib import exceptions
 from osc_lib import utils
 
